@@ -1,5 +1,4 @@
 import pickle
-import unittest
 
 import numpy
 import pytest
@@ -14,7 +13,12 @@ from cupy import cusparse
 from cupyx.scipy import sparse
 
 
-class TestMatDescriptor(unittest.TestCase):
+if cupy.cuda.runtime.is_hip:
+    pytest.skip('HIP sparse support is not yet ready',
+                allow_module_level=True)
+
+
+class TestMatDescriptor:
 
     def test_create(self):
         md = cusparse.MatDescriptor.create()
@@ -32,11 +36,12 @@ class TestMatDescriptor(unittest.TestCase):
     'transa': [True, False],
 }))
 @testing.with_requires('scipy')
-class TestCsrmm(unittest.TestCase):
+class TestCsrmm:
 
     alpha = 0.5
     beta = 0.25
 
+    @pytest.fixture(autouse=True)
     def setUp(self):
         self.op_a = scipy.sparse.random(2, 3, density=0.5, dtype=self.dtype)
         if self.transa:
@@ -73,11 +78,12 @@ class TestCsrmm(unittest.TestCase):
     'trans': [(False, False), (True, False), (False, True)],
 }))
 @testing.with_requires('scipy')
-class TestCsrmm2(unittest.TestCase):
+class TestCsrmm2:
 
     alpha = 0.5
     beta = 0.25
 
+    @pytest.fixture(autouse=True)
     def setUp(self):
         self.transa, self.transb = self.trans
         self.op_a = scipy.sparse.random(2, 3, density=0.5, dtype=self.dtype)
@@ -121,11 +127,12 @@ class TestCsrmm2(unittest.TestCase):
     'shape': [(3, 4), (4, 3)]
 }))
 @testing.with_requires('scipy>=1.2.0')
-class TestCsrgeam(unittest.TestCase):
+class TestCsrgeam:
 
     alpha = 0.5
     beta = 0.25
 
+    @pytest.fixture(autouse=True)
     def setUp(self):
         m, n = self.shape
         self.a = scipy.sparse.random(m, n, density=0.3, dtype=self.dtype)
@@ -151,11 +158,12 @@ class TestCsrgeam(unittest.TestCase):
 
 
 @testing.with_requires('scipy')
-class TestCsrgeamInvalidCases(unittest.TestCase):
+class TestCsrgeamInvalidCases:
 
     dtype = numpy.float32
     shape = (4, 3)
 
+    @pytest.fixture(autouse=True)
     def setUp(self):
         m, n = self.shape
         self.a = scipy.sparse.random(m, n, density=0.3, dtype=self.dtype)
@@ -166,9 +174,9 @@ class TestCsrgeamInvalidCases(unittest.TestCase):
             pytest.skip('csrgeam is not available')
         a = sparse.csc_matrix(self.a)
         b = sparse.csr_matrix(self.b)
-        with self.assertRaises(TypeError):
+        with pytest.raises(TypeError):
             cupy.cusparse.csrgeam(a, b)
-        with self.assertRaises(TypeError):
+        with pytest.raises(TypeError):
             cupy.cusparse.csrgeam(b, a)
 
     def test_csrgeam_invalid_shape(self):
@@ -176,14 +184,14 @@ class TestCsrgeamInvalidCases(unittest.TestCase):
             pytest.skip('csrgeam is not available')
         a = sparse.csr_matrix(self.a.T)
         b = sparse.csr_matrix(self.b)
-        with self.assertRaises(ValueError):
+        with pytest.raises(ValueError):
             cupy.cusparse.csrgeam(a, b)
 
     def test_csrgeam_availability(self):
         if not cupy.cusparse.check_availability('csrgeam'):
             a = sparse.csr_matrix(self.a)
             b = sparse.csr_matrix(self.b)
-            with self.assertRaises(RuntimeError):
+            with pytest.raises(RuntimeError):
                 cupy.cusparse.csrgeam(a, b)
 
     def test_csrgeam2_invalid_format(self):
@@ -191,9 +199,9 @@ class TestCsrgeamInvalidCases(unittest.TestCase):
             pytest.skip('csrgeam2 is not available')
         a = sparse.csc_matrix(self.a)
         b = sparse.csr_matrix(self.b)
-        with self.assertRaises(TypeError):
+        with pytest.raises(TypeError):
             cupy.cusparse.csrgeam2(a, b)
-        with self.assertRaises(TypeError):
+        with pytest.raises(TypeError):
             cupy.cusparse.csrgeam2(b, a)
 
     def test_csrgeam2_invalid_shape(self):
@@ -201,14 +209,14 @@ class TestCsrgeamInvalidCases(unittest.TestCase):
             pytest.skip('csrgeam2 is not available')
         a = sparse.csr_matrix(self.a.T)
         b = sparse.csr_matrix(self.b)
-        with self.assertRaises(ValueError):
+        with pytest.raises(ValueError):
             cupy.cusparse.csrgeam2(a, b)
 
     def test_csrgeam2_availability(self):
         if not cupy.cusparse.check_availability('csrgeam2'):
             a = sparse.csr_matrix(self.a)
             b = sparse.csr_matrix(self.b)
-            with self.assertRaises(RuntimeError):
+            with pytest.raises(RuntimeError):
                 cupy.cusparse.csrgeam2(a, b)
 
 
@@ -218,8 +226,9 @@ class TestCsrgeamInvalidCases(unittest.TestCase):
     'transb': [False, True],
 }))
 @testing.with_requires('scipy')
-class TestCsrgemm(unittest.TestCase):
+class TestCsrgemm:
 
+    @pytest.fixture(autouse=True)
     def setUp(self):
         self.op_a = scipy.sparse.random(2, 3, density=0.5, dtype=self.dtype)
         if self.transa:
@@ -248,11 +257,12 @@ class TestCsrgemm(unittest.TestCase):
     'shape': [(2, 3, 4), (4, 3, 2)]
 }))
 @testing.with_requires('scipy>=1.2.0')
-class TestCsrgemm2(unittest.TestCase):
+class TestCsrgemm2:
 
     alpha = 0.5
     beta = 0.25
 
+    @pytest.fixture(autouse=True)
     def setUp(self):
         m, n, k = self.shape
         self.a = scipy.sparse.random(m, k, density=0.5, dtype=self.dtype)
@@ -282,11 +292,12 @@ class TestCsrgemm2(unittest.TestCase):
 
 
 @testing.with_requires('scipy')
-class TestCsrgemm2InvalidCases(unittest.TestCase):
+class TestCsrgemm2InvalidCases:
 
     dtype = numpy.float32
     shape = (2, 3, 4)
 
+    @pytest.fixture(autouse=True)
     def setUp(self):
         m, n, k = self.shape
         self.a = scipy.sparse.random(m, k, density=0.5, dtype=self.dtype)
@@ -298,16 +309,16 @@ class TestCsrgemm2InvalidCases(unittest.TestCase):
             pytest.skip('csrgemm2 is not available.')
         a = sparse.csc_matrix(self.a)
         b = sparse.csr_matrix(self.b)
-        with self.assertRaises(TypeError):
+        with pytest.raises(TypeError):
             cupy.cusparse.csrgemm2(a, b)
         a = sparse.csr_matrix(self.a)
         b = sparse.csc_matrix(self.b)
-        with self.assertRaises(TypeError):
+        with pytest.raises(TypeError):
             cupy.cusparse.csrgemm2(a, b)
         a = sparse.csr_matrix(self.a)
         b = sparse.csr_matrix(self.b)
         d = sparse.csc_matrix(self.d)
-        with self.assertRaises(TypeError):
+        with pytest.raises(TypeError):
             cupy.cusparse.csrgemm2(a, b, d=d)
 
     def test_csrgemm2_invalid_shape(self):
@@ -315,23 +326,23 @@ class TestCsrgemm2InvalidCases(unittest.TestCase):
             pytest.skip('csrgemm2 is not available.')
         a = sparse.csc_matrix(self.a).T
         b = sparse.csr_matrix(self.b)
-        with self.assertRaises(ValueError):
+        with pytest.raises(ValueError):
             cupy.cusparse.csrgemm2(a, b)
         a = sparse.csr_matrix(self.a)
         b = sparse.csc_matrix(self.b).T
-        with self.assertRaises(ValueError):
+        with pytest.raises(ValueError):
             cupy.cusparse.csrgemm2(a, b)
         a = sparse.csr_matrix(self.a)
         b = sparse.csr_matrix(self.b)
         d = sparse.csc_matrix(self.d).T
-        with self.assertRaises(ValueError):
+        with pytest.raises(ValueError):
             cupy.cusparse.csrgemm2(a, b, d=d)
 
     def test_csrgemm2_availability(self):
         if not cupy.cusparse.check_availability('csrgemm2'):
             a = sparse.csr_matrix(self.a)
             b = sparse.csr_matrix(self.b)
-            with self.assertRaises(RuntimeError):
+            with pytest.raises(RuntimeError):
                 cupy.cusparse.csrgemm2(a, b)
 
 
@@ -340,11 +351,12 @@ class TestCsrgemm2InvalidCases(unittest.TestCase):
     'transa': [False, True],
 }))
 @testing.with_requires('scipy')
-class TestCsrmv(unittest.TestCase):
+class TestCsrmv:
 
     alpha = 0.5
     beta = 0.25
 
+    @pytest.fixture(autouse=True)
     def setUp(self):
         self.op_a = scipy.sparse.random(2, 3, density=0.5, dtype=self.dtype)
         if self.transa:
@@ -423,8 +435,9 @@ class TestCsrmv(unittest.TestCase):
 
 
 @testing.with_requires('scipy')
-class TestCoosort(unittest.TestCase):
+class TestCoosort:
 
+    @pytest.fixture(autouse=True)
     def setUp(self):
         if not cusparse.check_availability('coosort'):
             pytest.skip('coosort is not available')
@@ -454,8 +467,9 @@ class TestCoosort(unittest.TestCase):
 
 
 @testing.with_requires('scipy')
-class TestCsrsort(unittest.TestCase):
+class TestCsrsort:
 
+    @pytest.fixture(autouse=True)
     def setUp(self):
         if not cusparse.check_availability('csrsort'):
             pytest.skip('csrsort is not available')
@@ -476,8 +490,9 @@ class TestCsrsort(unittest.TestCase):
 
 
 @testing.with_requires('scipy')
-class TestCscsort(unittest.TestCase):
+class TestCscsort:
 
+    @pytest.fixture(autouse=True)
     def setUp(self):
         if not cusparse.check_availability('cscsort'):
             pytest.skip('cscsort is not available')
@@ -504,11 +519,12 @@ class TestCscsort(unittest.TestCase):
     'format': ['csr', 'csc', 'coo'],
 }))
 @testing.with_requires('scipy>=1.2.0')
-class TestSpmv(unittest.TestCase):
+class TestSpmv:
 
     alpha = 0.5
     beta = 0.25
 
+    @pytest.fixture(autouse=True)
     def setUp(self):
         m, n = self.shape
         self.op_a = scipy.sparse.random(m, n, density=0.5, format=self.format,
@@ -553,10 +569,11 @@ class TestSpmv(unittest.TestCase):
 
 
 @testing.with_requires('scipy')
-class TestErrorSpmv(unittest.TestCase):
+class TestErrorSpmv:
 
     dtype = numpy.float32
 
+    @pytest.fixture(autouse=True)
     def setUp(self):
         m, n = 2, 3
         self.a = scipy.sparse.random(m, n, density=0.5,
@@ -570,17 +587,17 @@ class TestErrorSpmv(unittest.TestCase):
 
         a = sparse.csr_matrix(self.a.T)
         x = cupy.array(self.x)
-        with self.assertRaises(ValueError):
+        with pytest.raises(ValueError):
             cupy.cusparse.spmv(a, x)
 
         a = sparse.csr_matrix(self.a)
         x = cupy.array(self.x)
-        with self.assertRaises(ValueError):
+        with pytest.raises(ValueError):
             cupy.cusparse.spmv(a, x, transa=True)
 
         a = sparse.csr_matrix(self.a)
         x = cupy.array(self.y)
-        with self.assertRaises(ValueError):
+        with pytest.raises(ValueError):
             cupy.cusparse.spmv(a, x)
 
 
@@ -592,11 +609,12 @@ class TestErrorSpmv(unittest.TestCase):
     'format': ['csr', 'csc', 'coo'],
 }))
 @testing.with_requires('scipy>=1.2.0')
-class TestSpmm(unittest.TestCase):
+class TestSpmm:
 
     alpha = 0.5
     beta = 0.25
 
+    @pytest.fixture(autouse=True)
     def setUp(self):
         m, n, k = self.dims
         self.op_a = scipy.sparse.random(m, k, density=0.5, format=self.format,
@@ -647,10 +665,11 @@ class TestSpmm(unittest.TestCase):
 
 
 @testing.with_requires('scipy')
-class TestErrorSpmm(unittest.TestCase):
+class TestErrorSpmm:
 
     dtype = numpy.float32
 
+    @pytest.fixture(autouse=True)
     def setUp(self):
         m, n, k = 2, 3, 4
         self.a = scipy.sparse.random(m, k, density=0.5,
@@ -664,26 +683,192 @@ class TestErrorSpmm(unittest.TestCase):
 
         a = sparse.csr_matrix(self.a.T)
         b = cupy.array(self.b, order='f')
-        with self.assertRaises(ValueError):
+        with pytest.raises(ValueError):
             cupy.cusparse.spmm(a, b)
 
         a = sparse.csr_matrix(self.a)
         b = cupy.array(self.b, order='f')
-        with self.assertRaises(AssertionError):
+        with pytest.raises(AssertionError):
             cupy.cusparse.spmm(a, b.T)
 
         a = sparse.csr_matrix(self.a)
         b = cupy.array(self.b)
-        with self.assertRaises(AssertionError):
+        with pytest.raises(AssertionError):
             cupy.cusparse.spmm(a, b)
 
         a = sparse.csr_matrix(self.a)
         b = cupy.array(self.c, order='f')
-        with self.assertRaises(ValueError):
+        with pytest.raises(ValueError):
             cupy.cusparse.spmm(a, b)
 
         a = sparse.csr_matrix(self.a)
         b = cupy.array(self.b, order='f')
         c = cupy.array(self.b, order='f')
-        with self.assertRaises(ValueError):
+        with pytest.raises(ValueError):
             cupy.cusparse.spmm(a, b, c=c)
+
+
+@testing.parameterize(*testing.product({
+    'lower': [True, False],
+    'unit_diag': [True, False],
+    'transa': ['N', 'T', 'H'],
+    'blocking': [True, False],
+    'level_info': [True, False],
+    'format': ['csr', 'csc'],
+    'nrhs': [None, 1, 4],
+    'order': ['C', 'F']
+}))
+@testing.with_requires('scipy')
+class TestCsrsm2:
+
+    n = 6
+    alpha = 1.0
+    density = 0.75
+    _tol = {'f': 1e-5, 'd': 1e-12}
+
+    def _setup(self, dtype):
+        dtype = numpy.dtype(dtype)
+        self.tol = self._tol[dtype.char.lower()]
+
+        a_shape = (self.n, self.n)
+        a = testing.shaped_random(a_shape, numpy, dtype=dtype, scale=1)
+        a_mask = testing.shaped_random(a_shape, numpy, dtype='f', scale=1)
+        a[a_mask > self.density] = 0
+        a_diag = numpy.diag(numpy.ones((self.n,), dtype=dtype))
+        if self.unit_diag:
+            a[a_diag > 0] = 0
+        a = a + a_diag
+        cp_a = cupy.array(a)
+        if self.unit_diag:
+            cp_a[a_diag > 0] = 0.1  # any number except 0
+        if self.format == 'csr':
+            self.a = sparse.csr_matrix(cp_a)
+        elif self.format == 'csc':
+            self.a = sparse.csc_matrix(cp_a)
+
+        b_shape = (self.n,) if self.nrhs is None else (self.n, self.nrhs)
+        b = numpy.arange(1, numpy.prod(b_shape) + 1,
+                         dtype=dtype).reshape(b_shape)
+        b = b.copy(order=self.order)
+        self.b = cupy.array(b, order=self.order)
+
+        if self.lower:
+            a = numpy.tril(a)
+        else:
+            a = numpy.triu(a)
+        if self.transa == 'T':
+            a = a.T
+        elif self.transa == 'H':
+            a = a.conj().T
+        self.ref_x = numpy.linalg.solve(a, self.alpha * b)
+
+    @testing.for_dtypes('fdFD')
+    def test_csrsm2(self, dtype):
+        if not cusparse.check_availability('csrsm2'):
+            pytest.skip('csrsm2 is not available')
+        if (self.format == 'csc' and numpy.dtype(dtype).char in 'FD' and
+                self.transa == 'H'):
+            pytest.skip('unsupported combination')
+        self._setup(dtype)
+        x = self.b.copy(order=self.order)
+        cusparse.csrsm2(self.a, x, alpha=self.alpha,
+                        lower=self.lower, unit_diag=self.unit_diag,
+                        transa=self.transa, blocking=self.blocking,
+                        level_info=self.level_info)
+        testing.assert_allclose(x, self.ref_x, atol=self.tol, rtol=self.tol)
+
+
+@testing.parameterize(*testing.product({
+    'n': [7, 10],
+    'level_info': [True, False],
+}))
+@testing.with_requires('scipy')
+class TestCsrilu02:
+
+    _tol = {'f': 1e-5, 'd': 1e-12}
+
+    def _make_matrix(self, dtype):
+        if not cusparse.check_availability('csrilu02'):
+            pytest.skip('csrilu02 is not available')
+        a = testing.shaped_random((self.n, self.n), cupy, dtype=dtype,
+                                  scale=0.9) + 0.1
+        a = a + cupy.diag(cupy.ones((self.n,), dtype=dtype.char.lower()))
+        return a
+
+    @testing.for_dtypes('fdFD')
+    def test_csrilu02(self, dtype):
+        dtype = numpy.dtype(dtype)
+        a_ref = self._make_matrix(dtype)
+        a = sparse.csr_matrix(a_ref)
+        cusparse.csrilu02(a, level_info=self.level_info)
+        a = a.todense()
+        al = cupy.tril(a, k=-1)
+        al = al + cupy.diag(cupy.ones((self.n,), dtype=dtype.char.lower()))
+        au = cupy.triu(a)
+        a = al @ au
+        tol = self._tol[dtype.char.lower()]
+        cupy.testing.assert_allclose(a, a_ref, atol=tol, rtol=tol)
+
+    def test_invalid_cases(self):
+        dtype = numpy.dtype('d')
+        a_ref = self._make_matrix(dtype)
+
+        # invalid format
+        a = sparse.csc_matrix(a_ref)
+        with pytest.raises(TypeError):
+            cusparse.csrilu02(a, level_info=self.level_info)
+
+        # invalid shape
+        a = cupy.ones((self.n, self.n + 1), dtype=dtype)
+        a = sparse.csr_matrix(a)
+        with pytest.raises(ValueError):
+            cusparse.csrilu02(a, level_info=self.level_info)
+
+        # matrix with zero diagonal element
+        a = a_ref
+        a[-1, -1] = 0
+        a = sparse.csr_matrix(a)
+        with pytest.raises(ValueError):
+            cusparse.csrilu02(a, level_info=self.level_info)
+
+        # singular matrix
+        a = a_ref
+        a[1:] = a[0]
+        a = sparse.csr_matrix(a)
+        with pytest.raises(ValueError):
+            cusparse.csrilu02(a, level_info=self.level_info)
+
+
+@testing.parameterize(*testing.product({
+    'shape': [(3, 4), (4, 4), (4, 3)],
+    'density': [0.0, 0.5, 1.0],
+    'format': ['csr', 'csc', 'coo']
+}))
+@testing.with_requires('scipy')
+class TestSparseMatrixConversion:
+
+    @testing.for_dtypes('fdFD')
+    def test_denseToSparse(self, dtype):
+        if not cusparse.check_availability('denseToSparse'):
+            pytest.skip('denseToSparse is not available')
+        x = cupy.random.uniform(0, 1, self.shape).astype(dtype)
+        x[x < self.density] = 0
+        y = cusparse.denseToSparse(x, format=self.format)
+        assert y.format == self.format
+        testing.assert_array_equal(x, y.todense())
+
+    @testing.for_dtypes('fdFD')
+    def test_sparseToDense(self, dtype):
+        if not cusparse.check_availability('sparseToDense'):
+            pytest.skip('sparseToDense is not available')
+        m, n = self.shape
+        x = scipy.sparse.random(m, n, density=self.density, format=self.format,
+                                dtype=dtype)
+        if self.format == 'csr':
+            x = sparse.csr_matrix(x)
+        elif self.format == 'csc':
+            x = sparse.csc_matrix(x)
+        elif self.format == 'coo':
+            x = sparse.coo_matrix(x)
+        y = cusparse.sparseToDense(x)
+        testing.assert_array_equal(x.todense(), y)
